{% load i18n %}
{% load gas_tags %}

{% if page %}
<nav class="pagination">
    <ul>
        {% if page.has_previous %}
        <li>
            <a href="?{% url_replace request page=page.previous_page_number %}">{% trans "Previous" %}</a>
        </li>
        {% endif %}
        <li class="current">
            {% blocktrans with number=page.number total=page.paginator.num_pages %}
            Page {{ number }} of {{ total }}
            {% endblocktrans %}
        </li>
        {% if page.has_next %}
            <li><a href="?{% url_replace request page=page.next_page_number %}">{% trans "Next" %}</a></li>
        {% endif %}
    </ul>

    <p>
        {% blocktrans with start=page.start_index end=page.end_index total_objects=page.paginator.count %}
        From {{ start }} to {{ end }} of {{ total_objects }} total items.
        {% endblocktrans %}
    </p>
</nav>
{% endif %}
